#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

import os
from unittest import TestCase, mock

from liminal.core.config.config import ConfigUtil


# noinspection PyUnresolvedReferences,DuplicatedCode
class TestHierarchicalConfig(TestCase):

    @mock.patch("liminal.core.util.files_util.load")
    def test_safe_load(self, find_config_files_mock):
        subliminal = {
            "name": "my_subliminal_test",
            "type": "sub",
            "super": "my_superliminal_test",
            "pipelines": [
                {"name": "mypipe1", "param": "constant"},
                {"name": "mypipe2", "param": "constant"}
            ],
            "pipeline_defaults": {
                "param1": "param1_value"
            },
            "task_defaults": {
                "job_start": {
                    "task_sub_def": "task_sub_def_value"
                }
            }
        }
        superliminal = {
            "name": "my_superliminal_test",
            "type": "super",
            "super": "super_superliminal",
            "pipeline_defaults": {
                "param2": "param2super_value",
                "param3": "param3super_value"
            },
            "task_defaults": {
                "job_start": {
                    "task_def1": "task_def1_value",
                    "task_def2": {
                        "task_def2_1": "task_def2_1_value",
                    }
                }
            }
        }
        super_superliminal = {
            "name": "super_superliminal",
            "type": "super",
            "pipeline_defaults": {
                "param2": "param2super_value",
                "param3": "param3hyper_value",
                "param4": "param4hyper_value"
            }
        }

        expected = [{'executors': [{'executor': 'default_k8s', 'type': 'kubernetes'}],
                     'name': 'my_subliminal_test',
                     'pipeline_defaults': {'param1': 'param1_value'},
                     'pipelines': [{'description': 'add defaults parameters for all pipelines',
                                    'name': 'mypipe1',
                                    'param': 'constant',
                                    'param1': 'param1_value',
                                    'param2': 'param2super_value',
                                    'param3': 'param3super_value',
                                    'param4': 'param4hyper_value',
                                    'tasks': [{'task': 'start',
                                               'task_def1': 'task_def1_value',
                                               'task_def2': {'task_def2_1': 'task_def2_1_value'},
                                               'task_sub_def': 'task_sub_def_value',
                                               'type': 'job_start'},
                                              {'task': 'end', 'type': 'job_end'}]},
                                   {'description': 'add defaults parameters for all pipelines',
                                    'name': 'mypipe2',
                                    'param': 'constant',
                                    'param1': 'param1_value',
                                    'param2': 'param2super_value',
                                    'param3': 'param3super_value',
                                    'param4': 'param4hyper_value',
                                    'tasks': [{'task': 'start',
                                               'task_def1': 'task_def1_value',
                                               'task_def2': {'task_def2_1': 'task_def2_1_value'},
                                               'task_sub_def': 'task_sub_def_value',
                                               'type': 'job_start'},
                                              {'task': 'end', 'type': 'job_end'}]}],
                     'service_defaults': {'description': 'add defaults parameters for all '
                                                         'services'},
                     'services': [],
                     'super': 'my_superliminal_test',
                     'task_defaults': {'job_start': {'task_sub_def': 'task_sub_def_value'}},
                     'type': 'sub'}]

        find_config_files_mock.return_value = {
            "my_subliminal_test": subliminal,
            "my_superliminal_test": superliminal,
            "super_superliminal": super_superliminal
        }

        config_util = ConfigUtil("")

        self.assertEqual(expected, config_util.safe_load(is_render_variables=True))

        # validate cache
        self.assertEqual(expected, config_util.loaded_subliminals)

    @mock.patch("liminal.core.util.files_util.load")
    def test_get_config(self, find_config_files_mock):
        find_config_files_mock.return_value = {
            "my_subliminal_test": {
                "type": "sub"
            },
            "my_superliminal_test": {
                "type": "super"
            }
        }

        config_util = ConfigUtil("")

        self.assertEqual({"type": "sub"},
                         config_util._ConfigUtil__get_config("my_subliminal_test"))

        self.assertEqual({"type": "super"},
                         config_util._ConfigUtil__get_config("my_superliminal_test"))

    @mock.patch("liminal.core.util.files_util.load")
    def test_get_superliminal(self, find_config_files_mock):
        base = {'executors': [{'executor': 'default_k8s', 'type': 'kubernetes'}],
                'name': 'base',
                'pipeline_defaults': {'after_tasks': [{'task': 'end', 'type': 'job_end'}],
                                      'before_tasks': [{'task': 'start', 'type': 'job_start'}],
                                      'description': 'add defaults parameters for all '
                                                     'pipelines'},
                'service_defaults': {'description': 'add defaults parameters for all '
                                                    'services'},
                'task_defaults': {'description': 'add defaults parameters for all tasks '
                                                 'separate by task type',
                                  'python': {'executor': 'default_k8s'}},
                'type': 'super'}
        subliminal = {
            "name": "subliminal_test",
            "type": "sub"
        }

        find_config_files_mock.return_value = {
            "subliminal_test": subliminal
        }

        config_util = ConfigUtil("")

        self.assertEqual(base,
                         config_util._ConfigUtil__get_superliminal(subliminal))

        self.assertEqual({},
                         config_util._ConfigUtil__get_superliminal(base))

        liminal = {
            "name": "subliminal_test",
            "type": "sub",
            "super": "my_superliminal"
        }

        with self.assertRaises(FileNotFoundError):
            config_util._ConfigUtil__get_superliminal(liminal)

    @mock.patch("liminal.core.util.files_util.load")
    def test_merge_superliminals(self, find_config_files_mock):
        superliminal = {
            "name": "my_superliminal_test",
            "type": "super",
            "super": "super_superliminal",
            "pipeline_defaults": {
                "before_tasks": [
                    {"task": "start-2", "type": "spark"},
                ],
                "after_tasks": [
                    {"task": "end-1", "type": "spark"}
                ]
            },
            "task_defaults": {
                "task_def1": "task_def1_value"
            }
        }

        super_superliminal = {
            "name": "super_superliminal",
            "type": "super",
            "pipeline_defaults": {
                "before_tasks": [
                    {"task": "start-1", "type": "spark"}],
                "after_tasks": [
                    {"task": "end-2", "type": "spark"}
                ]
            }
        }

        config_util = ConfigUtil("")

        find_config_files_mock.return_value = {
            "super_superliminal": super_superliminal,
            "superliminal": superliminal
        }

        expected = {'name': 'my_superliminal_test',
                    'pipeline_defaults': {'after_tasks': [{'task': 'end-1', 'type': 'spark'},
                                                          {'task': 'end-2', 'type': 'spark'}],
                                          'before_tasks': [{'task': 'start-1', 'type': 'spark'},
                                                           {'task': 'start-2', 'type': 'spark'}]},
                    'super': 'super_superliminal',
                    'task_defaults': {'task_def1': 'task_def1_value'},
                    'type': 'super'}

        self.assertEqual(expected,
                         dict(config_util._ConfigUtil__merge_superliminals(superliminal,
                                                                           super_superliminal)))

    @mock.patch("liminal.core.util.files_util.load")
    @mock.patch.dict(os.environ, {'env': 'myenv', 'LIMINAL_STAND_ALONE_MODE': 'True'})
    def test_safe_load_with_variables(self, find_config_files_mock):
        subliminal = {
            "name": "my_subliminal_test",
            "type": "sub",
            "super": "my_superliminal_test",
            "variables": {
                "var": "simple case",
                "var-2": "-case",
                "var_2": "_case",
                "image": "prod image",
                "a": "{{env}}1",
                "b": "{{a}}2",
                "c": "{{a}}{{b}}2"
            },
            "pipelines": [
                {"name": "mypipe1", "param": "{{var}}",
                 "tasks": [
                     {'task': 'sub_tasks',
                      'type': 'dummy'},
                 ]},
                {"name": "mypipe2", "param": "{{var-2   }}", "tasks": [
                    {'task': 'sub_tasks',
                     'type': 'dummy'},
                ]}
            ],
            "pipeline_defaults": {
                "param1": "{{var-2}}"
            },
            "task_defaults": {
                "job_start": {
                    "task_def1:": "task_sub_def_value"
                }

            },
            "services": [
                {
                    "name": "my_python_server",
                    "type": "python_server",
                    "image": "{{image}}"
                },
                {
                    "name": "my_python_server_for_stg",
                    "type": "python_server",
                    "image": "{{default_image}}"
                }
            ]}

        superliminal = {
            "name": "my_superliminal_test",
            "type": "super",
            "variables": {
                "var-2": "override",
                "var3": "super_var",
                "default_image": "default_image_value",
                "image": "default_image_value"
            },
            "super": "super_superliminal",
            "pipeline_defaults": {
                "param2": "{{pipe-var}}",
                "param3": "param3super_value",
                "before_tasks": [
                    {'task': 'second_task', 'type': 'dummy'},
                ]
            },
            "task_defaults": {
                "pipeline": {
                    "path": "{{var-2}}",
                    "task_def1": "task_def1_value",
                    "task_def2": {
                        "task_def2_1": "task_def2_1_value",
                    }
                }
            }
        }
        super_superliminal = {
            "name": "super_superliminal",
            "type": "super",
            "variables": {
                "default_image": "def_default_image_value"
            },
            "pipeline_defaults": {
                "global_conf": "{{var3}}",
                "param2": "param2super_value",
                "param3": "param3hyper_value",
                "param4": "param4hyper_value",
                "after_tasks": [
                    {'task': 'before_last_task', 'type': 'dummy'},
                ]
            }
        }

        expected = [{'executors': [{'executor': 'default_k8s', 'type': 'kubernetes'}],
                     'name': 'my_subliminal_test',
                     'pipeline_defaults': {'param1': '-case'},
                     'pipelines': [{'description': 'add defaults parameters for all pipelines',
                                    'global_conf': 'super_var',
                                    'name': 'mypipe1',
                                    'param': 'simple case',
                                    'param1': '-case',
                                    'param2': '{{pipe-var}}',
                                    'param3': 'param3super_value',
                                    'param4': 'param4hyper_value',
                                    'tasks': [{'task': 'start',
                                               'task_def1:': 'task_sub_def_value',
                                               'type': 'job_start'},
                                              {'task': 'second_task', 'type': 'dummy'},
                                              {'task': 'sub_tasks', 'type': 'dummy'},
                                              {'task': 'before_last_task', 'type': 'dummy'},
                                              {'task': 'end', 'type': 'job_end'}]},
                                   {'description': 'add defaults parameters for all pipelines',
                                    'global_conf': 'super_var',
                                    'name': 'mypipe2',
                                    'param': '-case',
                                    'param1': '-case',
                                    'param2': '{{pipe-var}}',
                                    'param3': 'param3super_value',
                                    'param4': 'param4hyper_value',
                                    'tasks': [{'task': 'start',
                                               'task_def1:': 'task_sub_def_value',
                                               'type': 'job_start'},
                                              {'task': 'second_task', 'type': 'dummy'},
                                              {'task': 'sub_tasks', 'type': 'dummy'},
                                              {'task': 'before_last_task', 'type': 'dummy'},
                                              {'task': 'end', 'type': 'job_end'}]}],
                     'service_defaults': {'description': 'add defaults parameters for all '
                                                         'services'},
                     'services': [{'description': 'add defaults parameters for all services',
                                   'image': 'prod image',
                                   'name': 'my_python_server',
                                   'type': 'python_server'},
                                  {'description': 'add defaults parameters for all services',
                                   'image': 'default_image_value',
                                   'name': 'my_python_server_for_stg',
                                   'type': 'python_server'}],
                     'super': 'my_superliminal_test',
                     'task_defaults': {'job_start': {'task_def1:': 'task_sub_def_value'}},
                     'type': 'sub',
                     'variables': {'a': 'myenv1',
                                   'b': 'myenv12',
                                   'c': 'myenv1myenv122',
                                   'image': 'prod image',
                                   'var': 'simple case',
                                   'var-2': '-case',
                                   'var_2': '_case'}}]

        find_config_files_mock.return_value = {
            "my_subliminal_test": subliminal,
            "my_superliminal_test": superliminal,
            "super_superliminal": super_superliminal
        }

        config_util = ConfigUtil("")

        self.assertEqual(expected, config_util.safe_load(is_render_variables=True))

        # validate cache
        self.assertEqual(expected, config_util.loaded_subliminals)

    @mock.patch('os.path.exists')
    @mock.patch("liminal.core.environment.get_airflow_home_dir")
    @mock.patch("liminal.core.util.files_util.load")
    @mock.patch.dict(os.environ, {'LIMINAL_STAND_ALONE_MODE': 'True', 'POD_NAMESPACE': 'my_pod_ns'})
    def test_liminal_config_snapshot(self, find_config_files_mock,
                                     get_airflow_dir_mock, path_exists_mock):
        subliminal = {
            "name": "my_subliminal_test",
            "type": "sub",
            "variables": {
                "var": 1,
                "var-2": True
            },
            "pipelines": [
                {"name": "mypipe1", "param": "{{var}}"},
                {"name": "mypipe2", "param": "{{var-2   }}"}
            ]
        }

        expected = {'name': 'my_subliminal_test', 'type': 'sub',
                    'executors': [{'executor': 'default_k8s', 'type': 'kubernetes'}],
                    'service_defaults': {'description': 'add defaults parameters for all services'},
                    'task_defaults': {
                        'description': 'add defaults parameters for all tasks separate by task type',
                        'python': {'executor': 'default_k8s'}}, 'pipeline_defaults': {
                'description': 'add defaults parameters for all pipelines',
                'before_tasks': [{'task': 'start', 'type': 'job_start'}],
                'after_tasks': [{'task': 'end', 'type': 'job_end'}]},
                    'variables': {'var': 1, 'var-2': True}, 'pipelines': [
                {'name': 'mypipe1', 'param': '1',
                 'description': 'add defaults parameters for all pipelines',
                 'tasks': [{'task': 'start', 'type': 'job_start'},
                           {'task': 'end', 'type': 'job_end'}]},
                {'name': 'mypipe2', 'param': 'True',
                 'description': 'add defaults parameters for all pipelines',
                 'tasks': [{'task': 'start', 'type': 'job_start'},
                           {'task': 'end', 'type': 'job_end'}]}], 'services': []}

        find_config_files_mock.return_value = {
            "my_subliminal_test": subliminal
        }

        get_airflow_dir_mock.return_value = "/tmp"
        path_exists_mock.return_value = True

        with mock.patch("builtins.open", mock.mock_open()) as m:
            with mock.patch("yaml.dump") as ydm:
                config_util = ConfigUtil("")
                config_util.safe_load(is_render_variables=True)
                config_util.snapshot_final_liminal_configs()

                m.assert_called_once_with(
                    os.path.join('/tmp', '../liminal_config_files/my_subliminal_test.yml'), 'w')
                ydm.assert_called_once_with(expected, m.return_value, default_flow_style=False)



#!/usr/bin/env python3
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

import logging
import os

import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

with open('requirements.txt') as f:
    requirements = f.read().splitlines()
    logging.info(requirements)

setuptools.setup(
    name="example-pkg-naturalett",
    version=os.environ.get("LIMINAL_BUILD_VERSION", os.environ.get('LIMINAL_VERSION', None)),
    author="dev@liminal.apache.org",
    author_email='dev@liminal.apache.org',
    description="A package for authoring and deploying machine learning workflows",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/apache/incubator-liminal",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    license='Apache License, Version 2.0',
    python_requires='>=3.6',
    install_requires=requirements,
    scripts=['scripts/liminal'],
    include_package_data=True,
    package_data={
        'liminal': ['../DISCLAIMER', '../LICENSE', '../NOTICE', '../README.md']
    }
)



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

import logging
import os
import sys

from kubernetes import client, config
from kubernetes.client import V1PersistentVolume, V1PersistentVolumeClaim

try:
    config.load_kube_config()
except:
    msg = "Kubernetes is not running\n"
    sys.stdout.write(f"INFO: {msg}")

_LOG = logging.getLogger('volume_util')
_LOCAL_VOLUMES = set([])
_kubernetes = client.CoreV1Api()


def create_local_volumes(liminal_config, base_dir):
    volumes_config = liminal_config.get('volumes', [])

    for volume_config in volumes_config:
        if 'local' in volume_config:
            logging.info(f'Creating local kubernetes volume if needed: {volume_config}')
            path = volume_config['local']['path']
            if path.startswith(".."):
                path = os.path.join(base_dir, path)
            if path.startswith("."):
                path = os.path.join(base_dir, path[1:])
            volume_config['local']['path'] = path
            create_local_volume(volume_config)


def create_local_volume(conf, namespace='default') -> None:
    name = conf['volume']

    _LOG.info(f'Requested volume {name}')

    if name not in _LOCAL_VOLUMES:
        matching_volumes = _kubernetes.list_persistent_volume(
            field_selector=f'metadata.name={name}'
        ).to_dict()['items']

        if not len(matching_volumes) > 0:
            _create_local_volume(conf, name)

        pvc_name = conf.get('claim_name', f'{name}-pvc')

        matching_claims = _kubernetes.list_persistent_volume_claim_for_all_namespaces(
            field_selector=f'metadata.name={pvc_name}'
        ).to_dict()['items']

        if not len(matching_claims) > 0:
            _create_persistent_volume_claim(pvc_name, name, namespace)

        _LOCAL_VOLUMES.add(name)


def delete_local_volume(name, namespace='default'):
    matching_volumes = _kubernetes.list_persistent_volume(
        field_selector=f'metadata.name={name}'
    ).to_dict()['items']

    if len(matching_volumes) > 0:
        _LOG.info(f'Deleting persistent volume {name}')
        _kubernetes.delete_persistent_volume(name)

    pvc_name = f'{name}-pvc'

    matching_claims = _kubernetes.list_persistent_volume_claim_for_all_namespaces(
        field_selector=f'metadata.name={pvc_name}'
    ).to_dict()['items']

    if len(matching_claims) > 0:
        _LOG.info(f'Deleting persistent volume claim {pvc_name}')
        _kubernetes.delete_namespaced_persistent_volume_claim(pvc_name, namespace)

    if name in _LOCAL_VOLUMES:
        _LOCAL_VOLUMES.remove(name)


def _create_persistent_volume_claim(pvc_name, volume_name, namespace):
    _LOG.info(f'Creating persistent volume claim {pvc_name} with volume {volume_name}')
    spec = {
        'volumeName': volume_name,
        'volumeMode': 'Filesystem',
        'storageClassName': 'local-storage',
        'accessModes': ['ReadWriteOnce'],
        'resources': {
            'requests': {
                'storage': '100Gi'
            }
        }
    }

    _kubernetes.create_namespaced_persistent_volume_claim(
        namespace,
        V1PersistentVolumeClaim(
            api_version='v1',
            kind='PersistentVolumeClaim',
            metadata={'name': pvc_name},
            spec=spec
        )
    )


def _create_local_volume(conf, name):
    _LOG.info(f'Creating persistent volume {name} with spec {conf}')
    spec = {
        'capacity': {
            'storage': '100Gi'
        },
        'volumeMode': 'Filesystem',
        'accessModes': ['ReadWriteOnce'],
        'persistentVolumeReclaimPolicy': 'Retain',
        'storageClassName': 'local-storage',
        'nodeAffinity': {
            'required': {
                'nodeSelectorTerms': [
                    {
                        'matchExpressions': [
                            {
                                'key': 'kubernetes.io/hostname',
                                'operator': 'NotIn',
                                'values': [
                                    ''
                                ]
                            }
                        ]
                    }
                ]
            }
        }
    }

    spec.update(conf)

    _kubernetes.create_persistent_volume(
        V1PersistentVolume(
            api_version='v1',
            kind='PersistentVolume',
            metadata={'name': name},
            spec=spec
        )
    )



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

import os
import unittest
from unittest import TestCase

import docker

from liminal.build import liminal_apps_builder

class TestLiminalAppsBuilder(TestCase):
    __image_names = [
        'my_python_task_img',
        'my_parallelized_python_task_img'
    ]

    def setUp(self) -> None:
        self.docker_client = docker.client.from_env()
        self.__remove_images()

    def tearDown(self) -> None:
        self.__remove_images()
        self.docker_client.close()

    def __remove_images(self):
        for image_name in self.__image_names:
            if len(self.docker_client.images.list(image_name)) > 0:
                self.docker_client.images.remove(image=image_name, force=True)

    def test_build_liminal(self):
        liminal_apps_builder.build_liminal_apps(
            os.path.join(os.path.dirname(__file__), '../liminal'))

        for image in self.__image_names:
            self.docker_client.images.get(image)


if __name__ == '__main__':
    unittest.main()



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

import logging
import os

import yaml

# { path -> { name -> config_file } }
cached_files = dict()
cached_source_files = dict()


def resolve_pipeline_source_file(config_name):
    """
    Return the source (file) path of the given config name
    """
    return cached_source_files[config_name]


def load(path):
    """
    :param path: config path
    :returns dict of {name -> loaded config file} from all liminal.y[a]ml files under given path
    """

    if cached_files.get(path):
        return cached_files[path]

    config_entities = {}

    for file_data in find_config_files(path):
        with open(file_data, 'r') as data:
            config_file = yaml.safe_load(data)
            config_entities[config_file['name']] = config_file
            cached_source_files[config_file['name']] = file_data

    cached_files[path] = config_entities
    return config_entities


def find_config_files(path):
    """
    :param path: config path
    :returns list of all liminal.y[a]ml files under config path
    """
    files = []
    logging.info(path)
    for r, d, f in os.walk(path):
        for file in f:
            if os.path.basename(file) in ['liminal.yml', 'liminal.yaml']:
                logging.info(os.path.join(r, file))
                files.append(os.path.join(r, file))
    return files


def dump_liminal_configs(liminal_configs, path):
    if not (os.path.exists(path)):
        os.mkdir(path)

    logging.info(f"Starting to dump liminal configs into {path}")

    for liminal_config in liminal_configs:
        dump_liminal_config(liminal_config, f'{path}/{liminal_config["name"]}.yml')


def dump_liminal_config(liminal_config, file_path):
    with open(file_path, 'w') as config_file:
        logging.info(f"Dumping {liminal_config['name']} into {file_path}")
        yaml.dump(liminal_config, config_file, default_flow_style=False)
